import os
import opie
import click
from helpers import u, op1, backups

@click.command()
def cli():
    backups.assert_environment()

    click.echo("backups found in " + backups.BACKUPS_DIR)
    archives = u.get_visible_children(backups.BACKUPS_DIR)
    i = 0
    for archive in archives:
        click.echo("%d. %s" % (i, archive))
        i += 1

    choice = click.prompt('Choose a backup', type=int)

    if choice < 0 or choice >= len(archives):
        exit("Invalid selection.")

    click.echo("you went with %d huh?" % (choice))

    mount = op1.get_mount_or_die_trying()
    print("Found at %s" % mount)

    click.echo("Restoring %s to %s" % (archives[choice], backups.BACKUPS_DIR))
    backups.restore_archive(os.path.join(backups.BACKUPS_DIR, archives[choice]), mount)




import os
import sys
import time
import opie
import click
import tarfile
import usb.core
import usb.util
from helpers import u
from datetime import datetime

BACKUPS_DIR = os.path.join(u.HOME, "backups")
ARCHIVE_FORMAT = "opie-backup-%Y-%m-%d-%H%M%S.tar.xz"

def assert_environment():
    os.makedirs(BACKUPS_DIR, exist_ok=True)

def generate_archive(mount, save_dir):
    name = os.path.join(save_dir, datetime.now().strftime(ARCHIVE_FORMAT))
    print("writing backup as %s" % (name))
    with tarfile.open(name, "x:xz") as tar:
        with click.progressbar(u.get_visible_children(mount)) as children:
            for child in children:
                tar.add(os.path.join(mount, child), child, exclude=lambda x: x[0] == '.')
    print("backup created.")

def restore_archive(file, mount):
    with tarfile.open(file, "r:xz") as tar:
        tar.extractall(path=mount)
    print("successfully restored from archive.")




import os
import sys
import time
import opie
import click
import tarfile
import usb.core
import usb.util
from helpers import u, mount

VENDOR_TE = 0x2367
PRODUCT_OP1 = 0x0002
OP1_BASE_DIRS = set(['tape', 'album', 'synth', 'drum'])

def ensure_connection():
    if not is_connected():
        print("Please connect your OP-1 and put it in DISK mode (Shift+COM -> 3)...")
        wait_for_connection()

def is_connected():
    return usb.core.find(idVendor=VENDOR_TE, idProduct=PRODUCT_OP1) is not None

def wait_for_connection():
    try:
        while True:
            time.sleep(1)
            if is_connected():
                break
    except KeyboardInterrupt:
        sys.exit(0)

def get_mount_or_die_trying():
    ensure_connection()
    mount = find_op1_mount()
    if mount is None:
        print("Waiting for OP-1 disk to mount...")
        mount = wait_for_op1_mount()
        if mount is None:
            exit("Failed to find mount point of OP-1. Feel free to file an issue.")
    return mount

def find_op1_mount():
    dirs = []
    mounts = mount.get_potential_mounts()
    if mounts is None:
        dirs = [os.path.join("/Volumes", x) for x in u.get_visible_folders("/Volumes")]
    else:
        dirs = [x[1] for x in mounts]

    for dir in dirs:
        subdirs = u.get_visible_folders(dir)
        if set(subdirs) & OP1_BASE_DIRS == OP1_BASE_DIRS:
            return dir

    return None

def wait_for_op1_mount(timeout=5):
    i=0
    try:
        while i < timeout:
            time.sleep(1)
            mount = find_op1_mount()
            if mount is not None:
                return mount
            i += 1
        print("timed out waiting for mount.")
        return None
    except KeyboardInterrupt:
        sys.exit(0)




import os
import sys
import time
import opie
import click
import tarfile
import usb.core
import usb.util
from datetime import datetime
from helpers import op1, rips

@click.command()
@click.option('--name', '-n', prompt=True)
def cli(name):
    rips.assert_environment()

    mount = op1.get_mount_or_die_trying()
    print("Found at %s" % mount)

    rips.create_rip(mount, name)



import os
import sys
import time
import opie
import click
import tarfile
import usb.core
import usb.util
from datetime import datetime
from helpers import op1, backups

@click.command()
def cli():
    backups.assert_environment()

    mount = op1.get_mount_or_die_trying()
    print("Found at %s" % mount)

    backups.generate_archive(mount, backups.BACKUPS_DIR)



import sys
import os
import re
from os import path
import subprocess
from subprocess import run

def get_mount_from_line(line):
    match = re.match(r"^\s*(?P<device>/dev/\w+) on (?P<mount>[^\0]+) (type .*)?\(.*\)\s*$", line)
    if match:
        return (match.group("device"), match.group("mount"))
    return None

def is_poopy_mount(mount):
    BAD_PREFIX = ['/dev', '/sys', '/net', '/proc', '/run', '/boot']
    if mount in ["/", "/home"]: return True
    for prefix in BAD_PREFIX:
        if mount.startswith(prefix): return True
    return False

def get_potential_mounts():
    result = run(["mount"], stdout=subprocess.PIPE, universal_newlines=True)
    if result.returncode != 0:
        print("mount command appeared to fail")
        return None

    if result.stdout is None:
        print("uh oh")

    lines = result.stdout.split("\n")
    mounts = [get_mount_from_line(x) for x in lines]
    filtered = [x for x in mounts if x is not None and not is_poopy_mount(x[1])]

    return filtered



import os
import sys
import time
import opie
import click
import shutil
import tarfile
import usb.core
import usb.util
from helpers import u
from datetime import datetime
from os import path
from subprocess import check_call

RIPS_DIR = os.path.join(u.HOME, "rips")

def assert_environment():
    os.makedirs(RIPS_DIR, exist_ok=True)

def transcode(input, codec, output, codec_flags=[]):
    binary = "ffmpeg"
    if shutil.which("ffmpeg") == None:
        if shutil.which("avconv") == None:
            click.echo("ffmpeg is required!")
            return
        else:
            binary = "avconv"

    base = [binary, "-loglevel", "warning","-stats"]
    args = base + \
        ["-i", input] + \
        ["-c:a", codec] + \
        codec_flags + \
        [output]
    click.echo(args)
    check_call(args)

def create_rip(mount, name):
    fullpath = path.join(RIPS_DIR, name)
    albums = path.join(mount, "album")
    sides = ["side_a", "side_b"]

    os.makedirs(fullpath, exist_ok=False)
    click.echo("writing rips to %s" % (fullpath))

    # initial transcode to lossless format
    for side in sides:
        click.echo("transcoding " + side)
        transcode(path.join(albums, side+".aif"), "flac", path.join(fullpath, side+".flac"))
    click.echo("ripped to lossless formats, now transcoding to some convenience formats.")
    for side in sides:
        origin = path.join(fullpath, side+".flac")
        click.echo("transcoding %s to alac".format(side))
        transcode(origin, "alac", path.join(fullpath, side+".m4a"))
        click.echo("transcoding %s to mp3 v0".format(side))
        transcode(origin, "libmp3lame", path.join(fullpath, side+".mp3"), ["-q:a", "0"])

    click.echo("ripped!")



import os
import hashlib
import configparser
from os import path

HOME = os.path.join(os.getenv("HOME"), "opie")
CONFIG_FILE = path.join(HOME, "opie.cfg")

def get_config():
    config = configparser.ConfigParser()
    config.read(CONFIG_FILE)
    return config

def write_config(config):
    with open(CONFIG_FILE, 'w') as configfile:
        config.write(configfile)

def get_visible_folders(d):
    return list(filter(lambda x: os.path.isdir(os.path.join(d, x)), get_visible_children(d)))

def get_visible_children(d):
    return list(filter(lambda x: x[0] != '.', os.listdir(d)))

def get_file_checksum(path):
    return hashlib.sha256(open(path, 'rb').read()).digest()

def dirtydiff(mount, source, target):
    skipped = 0
    copies = []
    for root, dirs, files in os.walk(os.path.join(mount, source), topdown=True):
        if "user" in dirs:
            del dirs[dirs.index("user")]
        for file in files:
            src = path.join(root, file)
            dst = path.join(target, path.relpath(src, mount))
            if not path.exists(dst) or get_file_checksum(src) != get_file_checksum(dst):
                copies.append((src, dst))
            else:
                skipped += 1
    return (copies, skipped)



import os
import opie
import click
import shutil
from urllib.parse import urlparse
from shutil import copytree, ignore_patterns
from os import path
from helpers import u, op1, backups
import tarfile
import requests
from requests.auth import HTTPBasicAuth

PACK_PATH = path.join(u.HOME, "packs")
EXPORT_PATH = path.join(u.HOME, "pkg")

def get_preset_path(mount, type, preset):
    return os.path.join(mount, type, "user", "%d.aif"%(preset))

@click.group()
def cli():
    pass

@click.command()
@click.option('--name', '-n', prompt=True)
@click.option('--group', '-g', default='opie', prompt=True)
@click.option('--type', '-t', type=click.Choice(['drum', 'synth']), prompt=True)
@click.option('--preset', '-p', type=click.IntRange(1, 8), prompt=True)
def make(name, group, type, preset):
    mount = op1.get_mount_or_die_trying()
    source = get_preset_path(mount, type, preset)
    dest_dir = os.path.join(mount, type, group)
    os.makedirs(dest_dir, exist_ok=True)

    dest = os.path.join(dest_dir, "%s.aif"%(name))
    if (os.path.exists(dest)):
        exit("that name is already taken in the group! be more creative!")

    shutil.copy(source, dest)
    click.echo("%s created!" % (dest))

@click.command()
def sync():
    mount = op1.get_mount_or_die_trying()

    copies = []
    skipped = 0
    for source in ["synth", "drum"]:
        (copy, skip) = u.dirtydiff(mount, source, PACK_PATH)
        copies += copy
        skipped += skip

    if len(copies) > 0:
        with click.progressbar(copies) as tasks:
            for task in tasks:
                os.makedirs(path.split(task[1])[0], exist_ok=True)
                shutil.copy2(task[0], task[1])

    click.echo("done! copied %d files, and skipped %d unchanged files." % (len(copies), skipped))


@click.command()
@click.option('--group', '-g', default='opie', prompt=True)
@click.option('--type', '-t', type=click.Choice(['drum', 'synth']), prompt=True)
def push(group, type):
    mount = op1.get_mount_or_die_trying()
    (copies, skipped) = u.dirtydiff(path.join(PACK_PATH, type), group, path.join(mount, type))

    if len(copies) > 0:
        with click.progressbar(copies) as tasks:
            for task in tasks:
                os.makedirs(path.split(task[1])[0], exist_ok=True)
                shutil.copy2(task[0], task[1])

    click.echo("done! skipped %d unchanged files." % (skipped))

@click.command()
@click.option('--group', '-g', default='opie', prompt=True)
@click.option('--type', '-t', type=click.Choice(['drum', 'synth']), prompt=True)
@click.option('--output-dir', '-o', default=EXPORT_PATH, prompt=True)
def export(group, type, output_dir):
    os.makedirs(output_dir, exist_ok=True)
    name = path.join(output_dir, "{}.tar.xz".format(group))
    source = path.join(PACK_PATH, type, group)
    with tarfile.open(name, "x:xz") as tar:
        tar.add(source, path.join(type, group), exclude=lambda x: x[0] == '.')
    click.echo("exported as %s" % name)

def get_user():
    config = u.get_config()
    if not 'auth' in config: return None
    return config['auth']['username']

def get_pass():
    config = u.get_config()
    if not 'auth' in config: return None
    return config['auth']['password']

@click.command()
@click.argument('input', type=click.Path(exists=True))
@click.option('--name', '-n', prompt=True)
@click.option('--description', '-d', prompt=True)
def upload(input, name, description):
    username = get_user()
    if username is None:
        username = click.prompt("username", type=str)
        password = click.prompt("password", type=str, hide_input=True)
        config = u.get_config()
        if not 'auth' in config: config['auth'] = {}
        config['auth']['username'] = username
        config['auth']['password'] = password
        u.write_config(config)
    else:
        password = get_pass()
    url = 'http://localhost:5000/upload'
    files = [('file', open(input, 'rb'))]
    auth = HTTPBasicAuth(username, password)
    metadata = {
        'name': name,
        'description': description,
    }

    r = requests.post(url, files=files, data=metadata, auth=auth, allow_redirects=False)
    click.echo(r.status_code)
    click.echo(r.headers['Location'])

@click.command()
@click.argument('url')
def download(url):
    click.echo("downloading {}".format(url))
    fname = path.basename(urlparse(url).path)
    fullpath = path.join(EXPORT_PATH, fname)
    if path.exists(fullpath):
        click.echo("already downloaded!")
    else:
        r = requests.get(url)
        with open(fullpath, 'wb') as f:
            f.write(r.content)

        click.echo("saved to {}".format(fullpath))
    imp(fullpath)

def imp(file):
    if not path.exists(file):
        click.echo("File doesn't exist.")

    with tarfile.open(file, "r:xz") as tar:
        members = [x for x in tar.getmembers()
                   if (x.isdir() and path.split(x.name[:-1])[0] in ['drum','synth'])
                   or x.name[-4:] == '.aif']

        tar.extractall(PACK_PATH, members)
        for member in members:
            click.echo(member.name)

        click.echo("\ndone!")

@click.command()
@click.argument("file")
def add(file):
    imp(file)

@click.command()
def list():
    synths = os.listdir(path.join(PACK_PATH, "synth"))
    drums = os.listdir(path.join(PACK_PATH, "drum"))
    click.echo("SYNTH")
    for synth in synths:
        if not synth[0] == '.':
            click.echo("  %s" % synth)
    click.echo("DRUM")
    for drum in drums:
        if not drum[0] == '.':
            click.echo("  %s" % drum)

cli.add_command(make)
cli.add_command(sync)
cli.add_command(push)
cli.add_command(export)
cli.add_command(add)
cli.add_command(list)
cli.add_command(upload)
cli.add_command(download)




import os
import sys
import opie
import click
import shutil
import subprocess
from subprocess import run
from helpers import u, op1, backups

@click.command()
def cli():
    if sys.platform != "darwin":
        exit("Eject only supported on OS X for now, sorry.")

    if not op1.is_connected():
        exit("OP-1 doesn't appear to be connected.")
    mount = op1.find_op1_mount()
    if mount is None:
        exit("Looks like your best friend already dismounted.")

    click.echo("attempting to eject...")
    click.echo(run(["diskutil", "eject", mount], stderr=subprocess.STDOUT).stdout)



from setuptools import setup

setup(
    name='opie',
    version='0.0.7',
    author='Jake McGinty',
    author_email='me@jake.su',
    description=("A helping friend for the OP-1 synthesizer."),
    py_modules=['opie'],
    packages=['commands', 'helpers'],
    install_requires=[
        'Click',
        'pyusb',
        'requests',
        'colorama',
    ],
    entry_points='''
        [console_scripts]
        opie=opie:cli
    ''',
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Topic :: Utilities",
    ]
)
